static char *sccsid = "@(#) lib/exits/oltp/pc/os2/amqzsc52.c, lqm, p000, p000-L951027 	1.3 95/10/20 10:10:18";
/*********************************************************************/
/*                                                                   */
/* Module Name: amqzsc52.c                                           */
/*                                                                   */
/* Description: MQSeries for OS/2 exit program for CICS/2            */
/*              User Exit 15/16 - Task Attach/Detach                 */
/*                                                                   */
/*  Statement:     Licensed Materials - Property of IBM              */                      
/*                                                                   */      
/*                 84H2000, 5765-B73                                 */            
/*                 84H2001, 5639-B42                                 */            
/*                 84H2002, 5765-B74                                 */            
/*                 84H2003, 5765-B75                                 */            
/*                 84H2004, 5639-B43                                 */            
/*                 (C) Copyright IBM Corp. 1994, 1997                */  
/*********************************************************************/
/*                                                                   */
/* Function:                                                         */
/*                                                                   */
/* amqzsc52.c is the Task Attach/Detach Exit program for CICS/2.     */
/* It is based on the CICS/2 sample code FAAEXP15.C, whose original  */
/* header and copyright is included below.                           */
/*                                                                   */
/* amqzsc52.c contains the following functions:                      */
/*  FAAEXP15             CICS/2 User entry point 15/16               */
/*                                                                   */
/*********************************************************************/


/***********************************************************************/
/*                                                                     */
/* MODULE NAME        FAAEXP15.C                                       */
/*                                                                     */
/* DESCRIPTIVE NAME   User exit - 15. Task Attach / Detach Exit        */
/*                                                                     */
/* Statement:         Licensed Materials - Property of IBM             */
/*                                                                     */
/*                    53G3861,53G3862                                  */
/*                    (c) Copyright IBM Corp. 1988, 1993.              */
/*                                                                     */
/*                    See Copyright Instructions.                      */
/*                                                                     */
/*                    All rights reserved.                             */
/*                                                                     */
/*                    U.S. Government Users Restricted Rights - use,   */
/*                    duplication or disclosure restricted by GSA      */
/*                    ADP Schedule Contract with IBM Corp.             */
/*                                                                     */
/* Status:            Version 2 Release 0                              */
/*                                                                     */
/*  NOTES :-                                                           */
/*    DEPENDENCIES = None                                              */
/*                   None                                              */
/*    RESTRICTIONS = none                                              */
/*    MODULE TYPE  = Program                                           */
/*    PROCESSOR    = PS/2 and PC                                       */
/*                                                                     */
/***********************************************************************/
/*                                                                     */
/*  FUNCTION:                                                          */
/*       To enable the user to perform any non CICS API calls at task  */
/*       attach or detach time.  The parameters passed to this exit    */
/*       program are defined in the header FAAEXH15.H   Task attach is */
/*       identified by the exit number being set to 16, while for task */
/*       detach the value is set to 15.  These values are defined in   */
/*       the referenced copybook.                                      */
/*                                                                     */
/***********************************************************************/

/*********************************************************************/
/* Includes                                                          */
/*********************************************************************/

/*********************************************************************/
/* Select compiler specific header files                             */
/*********************************************************************/

#include <string.h>

/*********************************************************************/
/* Select CICS/2 header files                                        */
/*********************************************************************/

#include <faaexh15.h>

/*********************************************************************/
/* Select MQI header file                                            */
/*********************************************************************/

#include <cmqc.h>

/*********************************************************************/
/* Define the MQ externals for this function                         */
/*********************************************************************/

extern void FAAFN1616 AMQ1PHASETASKATTACH  ( void );
extern void FAAFN1616 AMQ1PHASETASKDETACH  ( void );
extern void FAAFN1616 AMQ1PHASETASKABEND   ( void );
extern void FAAFN1616 AMQCICSINIT(faap_exit_15_t pExit15Input);

/*********************************************************************/
/* Data defined in this module                                       */
/*********************************************************************/

static char dummy;

/*********************************************************************/
/* Local function declarations                                       */
/*********************************************************************/

/*********************************************************************/
/* Function Definitions                                              */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name:  FAAEXP15                                          */
/*                                                                   */
/* Description: CICS/2 User Exit 15 - Task detach.                   */
/*                                                                   */
/*********************************************************************/
/* Following text will be included in the Service Reference Manual . */
/* Ensure that the content is correct and up-to-date.                */
/* All updates should be made in mixed case.                         */
/*                                                                   */
/* Function:                                                         */
/*                                                                   */
/*  FAAEXP15 is called by CICS/2 whenever a user task attaches       */
/*  or detaches, except for a detach caused by a system abend.       */
/*  This latter case is handled by CICS User Exit 17.                */
/*                                                                   */
/*  For Task Attach:-                                                */
/*                                                                   */
/*  We must notify MQ that the calling task is running under the     */
/*  control of CIC/2 and that MQ must therefore register with CICS/2 */
/*  in order to perform any SyncPoint operations. MQ is notified     */
/*  via a call to the function AMQCICSINIT, in library MQMCICS       */
/*  Additionally MQ is called via function AMQ1PHASEATTACH at the    */
/*  start of every transaction. The above calls disable the          */
/*  MQCMIT and MQBACK calls.                                         */
/*                                                                   */
/*  For Task Detach:-                                                */
/*                                                                   */
/*  Since the task may still be connected, and have Open Queues      */
/*  we should perform any MQDISC processing that should have been    */
/*  performed by the CICS task before it terminated. CICS tells      */
/*  us in the Exit15TermStatus field of the passed parameter         */
/*  block whether to commit or rollback any active CICS transactions */
/*  MQ is notified as to the fact that the task has detached, and    */
/*  of the outcome of any existing transaction by a call to either   */
/*  AMQ1PHASETASKDETACH, or AMQ1PHASETASKABEND, both in library      */
/*  MQMCICS, as appropriate.                                         */
/*                                                                   */
/* Input Parameters:  pExit15Input (faap_exit_15_t)                  */
/*                     - Exit 15 parameters                          */
/*                                                                   */
/* Output Parameters: None                                           */
/*                                                                   */
/* InOut Parameters:  None                                           */
/*                                                                   */
/* Returns:           * indicates FFST has been performed            */
/*     0                          - Success.                         */
/*                                                                   */
/* Exit Normal:       See Above                                      */
/*                                                                   */
/* Exit Error:        See Above                                      */
/*                                                                   */
/* End of text to be included in SRM                                 */
/*********************************************************************/
/*                                                                   */
/* Notes:                                                            */
/*   Restrictions: None                                              */
/*                                                                   */
/*********************************************************************/
faaret_t FAAEXPENTRY FAAEXP15(faap_exit_15_t pExit15Input)
{
faaret_t rc = 0;

  /*******************************************************************/
  /* If the task is attaching                                        */
  /*******************************************************************/
  if (pExit15Input->ExitNumber == EXIT_15_TASK_ATTACH)
  {
    /*****************************************************************/
    /* Initialise MQ CICS support                                    */
    /*****************************************************************/
    AMQCICSINIT(pExit15Input);

    /*****************************************************************/
    /* Attach the MQSeries session to CICS/2                         */
    /*****************************************************************/
    AMQ1PHASETASKATTACH();

  /*******************************************************************/
  /* else if the task is detaching                                   */
  /*******************************************************************/
  } else if (pExit15Input->ExitNumber == EXIT_15_TASK_DETACH)
  {
    /*****************************************************************/
    /* If the transaction has aborted                                */
    /*****************************************************************/
    if (pExit15Input->Exit15TermStatus == EXIT_15_TRAN_ABENDED)
    {
      /***************************************************************/
      /* Detach and abort any active CICS transactions               */
      /***************************************************************/
      AMQ1PHASETASKABEND();

    /*****************************************************************/
    /* else the transaction has committed                            */
    /*****************************************************************/
    } else if (pExit15Input->Exit15TermStatus
                    == EXIT_15_TRAN_NORMAL_END)
    {
      /***************************************************************/
      /* Detach and commit any active CICS transactions              */
      /***************************************************************/
      AMQ1PHASETASKDETACH();
    };
  };

  /*******************************************************************/
  /* Return from function                                            */
  /*******************************************************************/
  return rc;

} /* End function FAAEXP15                                           */

/*********************************************************************/
/*                                                                   */
/* End of amqzsc52.c                                                 */
/*                                                                   */
/*********************************************************************/
