static char *sccsid = "@(#) lib/exits/oltp/pc/winnt/amqzscgn.c, lqm, p000, p000-L981126            1.6   98/11/25 17:40:07";
/*********************************************************************/
/*                                                                   */
/* Module Name: amqzscgn.c                                           */
/*                                                                   */
/* Description: MQSeries sample CICS GLobal User Exit (GLUE) program */
/*              for the CICS Task termination user exit (UE014015)   */
/*                                                                   */
/* Statement:   Licensed Materials - Property of IBM                 */
/*                                                                   */
/* 84H2004, 5639-B43                                                 */
/* (C) Copyright IBM Corp. 1996, 1997                                */
/* All Rights Reserved                                               */
/*                                                                   */
/* US Government Users Restricted Rights - Use, duplication or       */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp. */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Function:                                                         */
/*                                                                   */
/* This sample is the MQSeries CICS Task termination user exit       */
/* (UE014015) program.                                               */
/*                                                                   */
/* It is based on the samples in the CICS documentation, and         */
/* contains the following function:                                  */
/*  cics_UE_entry  -  CICS UE entry                                  */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Use the cl command to build amqzscgn.obj by compiling using at    */
/* least the following:                                              */
/* cl -c -I<YourCicsPath>\include -Gz -LD amqzscgn.c                 */
/*                                                                   */
/* Create a module definition file named mqmc1415.def containing     */
/* 3 lines of:                                                       */
/*   LIBRARY MQMC1415                                                */
/*   EXPORTS                                                         */
/*   cics_UE_entry=cics_UE_entry@8                                   */
/*                                                                   */  
/* Use the lib command to build an export file & import library      */
/* using at least the following:                                     */  
/* lib -def:mqmc1415.def -out:mqmc1415.lib                           */
/*                                                                   */
/* If the lib command is successful then an mqmc1415.exp will also   */
/* have been built.                                                  */
/*                                                                   */
/* Use the link command to build mqmc1415.dll by linking using at    */
/* least the following:                                              */
/* link -dll -nod -out:mqmc1415.dll amqzscgn.obj mqmc1415.exp        */
/*   mqmcics4.lib msvcrt.lib kernel32.lib                            */
/*                                                                   */
/*********************************************************************/

/*********************************************************************/
/* Includes                                                          */
/*********************************************************************/

/*********************************************************************/
/* Select compiler specific header files                             */
/*********************************************************************/

#include <stdio.h>
#include <errno.h>

/*********************************************************************/
/* Select CICS include files                                         */
/*********************************************************************/

#include <cicstype.h>
#include <cicsue.h>

/*********************************************************************/
/* Select MQI header file                                            */
/*********************************************************************/

#include <cmqc.h>

/*********************************************************************/
/* Data defined in this module                                       */
/*********************************************************************/

/*********************************************************************/
/* Local function declarations                                       */
/*********************************************************************/
extern void MQENTRY AMQ2PHASETASKDETACH(void);
extern void MQENTRY AMQ2PHASETASKABEND(void);

/*********************************************************************/
/* Function Definitions                                              */
/*********************************************************************/

/*********************************************************************/
/*                                                                   */
/* Function Name:  cics_UE_entry                                     */
/*                                                                   */
/* Description: CICS Task termination user exit (UE014015)           */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Function:                                                         */
/*                                                                   */
/*  When the task termination exit number 15 is enabled on CICS,     */
/*  CICS calls cics_UE_entry whenever a task terminates.             */
/*                                                                   */
/* Input Parameters:  UE_Header(cics_UE_Header_t *)                  */
/*                    - Exit generic information                     */
/*                                                                   */
/*                    UE_Specific(cics_UE014015_t *)                 */
/*                    - Exit specific information                    */
/*                                                                   */
/* Output Parameters: None                                           */
/*                                                                   */
/* InOut Parameters:  None                                           */
/*                                                                   */
/* Returns:           UE_HeaderVersion                               */
/*                    UE_SpecificVersion                             */
/*                    UE_Normal                                      */
/*                                                                   */
/*********************************************************************/
cics_UE_Return_t cics_UE_entry(cics_UE_Header_t *UE_Header,
                               cics_UE014015_t  *UE_Specific)
{
 cics_UE_Return_t rc = UE_HeaderVersion;

 /********************************************************************/
 /* Check we understand the versions of the structures               */
 /********************************************************************/
 if (UE_Header->UE_Version == cics_UE_HEADER_VERSION)
 {
    rc = UE_SpecificVersion;

    if (UE_Specific->UE_Version == cics_UE014015_VERSION)
    {
     /****************************************************************/
     /* Determine the type of task termination                       */
     /****************************************************************/
     if (UE_Specific->UE_Terminationtype == UE_Normaltermination)
     {
       /**************************************************************/
       /* Normal termination - inform MQ.                            */
       /* MQ will take the appropriate action including performing   */
       /* an MQDISC.                                                 */
       /**************************************************************/
       AMQ2PHASETASKDETACH();
     }
     else
     {
       /**************************************************************/
       /* Abnormal termination - inform MQ.                          */
       /* MQ will take the appropriate action including performing   */
       /* an MQDISC.                                                 */
       /**************************************************************/
       AMQ2PHASETASKABEND();
     }

     rc = UE_Normal;
    }
 }

 return rc;
}
/*********************************************************************/
/*                                                                   */
/* End of amqzscgn.c                                                 */
/*                                                                   */
/*********************************************************************/
